/* -*- mode:c;  c-basic-offset: 4; tab-width: 8; -*- vi: set sw=4 ts=8:*/
/** 
 * @file
 * Declaration of SIXNET IODB functions
 *
 * Copyright 2009 SIXNET, LLC.  All rights reserved.
 */

#ifndef SX_IODB_H
#define SX_IODB_H
/*
 *
 *  iodb.h: Last Revised: 1/30/02
 *
 *
 *  This is the include file containing the export 
 *  functions representing the published user application
 *  32-bit I/O database
 *
 */

#include "utypes.h"
#include <time.h>

#define IODB_VERSION	0x0100  // High byte:major, Low Byte: Minor

/* function return values: */
#define EDBERROR          -1
#define ENOERROR           0
#define EOUTOFRANGE        1
#define EINVALIDTYPE       2
#define ENOSTATION         3
#define EFILEACCESS        4
#define EFILENOTFOUND      5
#define EIODBNOTFOUND      6
#define EOUTOFMEMORY       7
#define ETASKERROR         8
#define ENONSIXNETFILE     9
#define ETAGNOTFOUND      10
#define EDUPLICATETAG     11
#define EBUFFERSIZE       12
#define ENOTANALOGTYPE    13
#define EREGISTERNOTFOUND 14
#define EEXCEEDEDMAXTAGS  15

/* Analog Formats: */
#define NOFORMAT           1
#define SFORMAT            2
#define UFORMAT            3
#define LFORMAT            4
#define ULFORMAT           5

/* I/O Data Types: */
#define RESERVEDtype  0
#define ANALOGtype    1
#define DISCRETEtype  2
#define BYTEtype      3
#define LONGtype      4
#define FLOATtype     5
#define DOUBLEtype    6
#define USERtype      7

/* Update Actions: */
#define READSCAN    0
#define WRITESCAN   1
#define WRITEREAD   2
#define EXCEPTION   3
#define ASSIGNED    4

/* Station Types: */
#define SIXTRAK      0
#define IOMUX        1
#define VERSAMUX     2
#define S60IBMN      3
#define LOCALCOMP    4
#define IODB         5
#define MISCTYPE     6
#define VERSATRAK    7
#define REMOTETRAK   8
#define ETHERTRAK    9

/* Software Packages: */
#define SYSPLAN_PROG 0
#define SPF_PROG     1
#define SCR_PROG     2
#define SCRADV_PROG  3
#define SCR_1131     4
#define SCR_1131_ADV 5

/* Registration Types: */
#define NO_LICENSE        -1
#define TEMPORARY_LICENSE  0 
#define DEMO_LICENSE       1
#define SINGLE_LICENSE     2
#define SITE_LICENSE       3
#define OEM_LICENSE        4

#define BYTE unsigned char
#define USHORT unsigned short
#define IODBerr unsigned short

/* Support Functions */
IODBerr IODBCleanup(void);

/* I/O Updating Functions */
IODBerr IODBSetTag(const char *, USHORT, USHORT, USHORT);
IODBerr IODBRead(USHORT, USHORT, USHORT, void *, const char * );
IODBerr IODBReadTag(const char  *, void *, USHORT, USHORT  *, USHORT * );
IODBerr IODBWrite(USHORT, USHORT, USHORT, void  *, const char * );
IODBerr IODBWriteMask(USHORT, USHORT, USHORT, void *, BYTE *, const char * );
IODBerr IODBWriteTag(const char *, void * );
IODBerr IODBGetNextTag(short *, char *, char *, char *, USHORT *,
                       USHORT *, USHORT *, USHORT *);

/* I/O Status Functions */
IODBerr IODBGetDescription(const char *, USHORT, USHORT, char *);
IODBerr IODBGetFormat(const char *, USHORT, USHORT, BYTE *);
IODBerr IODBMinMax(const char *, USHORT, USHORT, long *, long *,
                   double *, double *, char *);
IODBerr IODBMinMaxTag(const char *, long *, long *, double *,
                      double *, char *);
IODBerr IODBScale(const char *, USHORT, USHORT, double *, double *, char *);
IODBerr IODBScaleTag(const char *, double *, double *, char *);

/* Advanced Driver Functions */
void    IODBGetFile(char *, char *, time_t *);
IODBerr IODBGetNextTag(short *, char *, char *, char *, USHORT *,
                       USHORT *, USHORT *, USHORT *);
IODBerr IODBGetNextType(short *, USHORT *, USHORT *, USHORT *, USHORT *,
                        char *, const char *);
IODBerr IODBGetTag(const char *, char *, USHORT *, USHORT *, USHORT *,
                   USHORT *, USHORT *);
IODBerr IODBGetType(USHORT, USHORT *, USHORT *, USHORT *, char *);
IODBerr IODBGetTypeRange(USHORT, const char *, USHORT *, USHORT *);
USHORT IODBVersion(void);

#endif /* SX_IODB_H */
