/*
 *  Copyright (C) 2016 Red Lion Controls, Inc.
 *
 *  ThingWorx-SN_SDK Example Program
 */

#include "twOSPort.h"
#include "twLogger.h"
#include "twApi.h"

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <unistd.h>
#include <iodb.h>
#include "configs.h"


/* Name of our thing */

char thingName[32];


/* Types */
#define INT16   short
#define INT32   long
#define UINT16  unsigned short
#define UINT32  unsigned long
#define ULONG   unsigned long
#define BYTE    unsigned char
#define BOOL    int

/* IODB Types */
#define ANALOG_IN       0
#define ANALOG_OUT      1
#define DISCRETE_IN     10
#define DISCRETE_OUT    11
#define LONG_IN         20
#define LONG_OUT        21
#define FLOAT_IN        22
#define FLOAT_OUT       23


/*****************
A simple structure to handle
properties. Not related to
the API in anyway, just for
the demo application.
******************/
struct  {
    double PowerSupply;
    int Temperature;
    int TimeHours;
    int TimeMinutes;
    int TimeSeconds;
    double Potentiameter;
    char LedOn;
    char Led2On;
} properties;

INT16 addr;
INT16 regs;
INT16 ledOnRead;
INT16 ecode;
/*****************
Helper Functions
*****************/
// sendPropertyUpdate - send properties to ThingWorx Server
void sendPropertyUpdate() {
    /* Create the property list */
    propertyList * proplist = twApi_CreatePropertyList("PowerSupply",twPrimitive_CreateFromNumber(properties.PowerSupply), 0);
    if (!proplist) {
        TW_LOG(TW_ERROR,"sendPropertyUpdate: Error allocating property list");
        return;
    }
    twApi_AddPropertyToList(proplist,"LedOn",twPrimitive_CreateFromBoolean(properties.LedOn), 0);
    twApi_AddPropertyToList(proplist,"Led2On",twPrimitive_CreateFromBoolean(properties.Led2On), 0);
    twApi_AddPropertyToList(proplist,"Temperature",twPrimitive_CreateFromInteger(properties.Temperature), 0);
    twApi_AddPropertyToList(proplist,"TimeHours",twPrimitive_CreateFromInteger(properties.TimeHours), 0);
    twApi_AddPropertyToList(proplist,"TimeMinutes",twPrimitive_CreateFromInteger(properties.TimeMinutes), 0);
    twApi_AddPropertyToList(proplist,"TimeSeconds",twPrimitive_CreateFromInteger(properties.TimeSeconds), 0);
    twApi_AddPropertyToList(proplist,"Potentiameter",twPrimitive_CreateFromNumber(properties.Potentiameter), 0);
    twApi_PushProperties(TW_THING, thingName, proplist, -1, FALSE);
    twApi_DeletePropertyList(proplist);
}



/***************
Data Collection Task
****************/
/*
This function gets called at the rate defined in the task creation.  The SDK has 
a simple cooperative multitasker, so the function cannot infinitely loop.
Use of a task like this is optional and not required in a multithreaded
environment where this functonality could be provided in a separate thread.
*/
#define DATA_COLLECTION_RATE_MSEC 500
void dataCollectionTask(DATETIME now, void * params) {

    int pRead = 0;
    double pd;

    IODBRead(ANALOG_IN, 61, 1, (void *)&pRead, NULL);
    pd = pRead;
    properties.PowerSupply = pd/1000;
    int oldSec = properties.TimeSeconds;

    IODBRead(ANALOG_IN, 60, 1, (void *)&properties.Temperature, NULL);
    IODBRead(ANALOG_OUT, 1015, 1, (void *)&properties.TimeHours, NULL);
    IODBRead(ANALOG_OUT, 1016, 1, (void *)&properties.TimeMinutes, NULL);
    IODBRead(ANALOG_OUT, 1017, 1, (void *)&properties.TimeSeconds, NULL);
    IODBRead(ANALOG_IN, 0, 1, (void *)&pRead, NULL);
    pd = pRead;
    properties.Potentiameter = pd/1000;

    ledOnRead = 0;
    ecode = IODBRead(DISCRETE_OUT, 1, 1, (void *)&ledOnRead, NULL);

    if(ledOnRead == 0)
    {properties.LedOn = FALSE;}
    else
    {properties.LedOn = TRUE;}


    // Change Led 2 based on state
    if(properties.Led2On == TRUE)// Led is off, turn it on
    {
        ledOnRead = 1;
        ecode = IODBWrite(DISCRETE_OUT, 0, 1, (void *)&ledOnRead, NULL);
    }
    else // turn  led off
    {
        ledOnRead = 0;
        ecode = IODBWrite(DISCRETE_OUT, 0, 1, (void *)&ledOnRead, NULL);
    }

    if(oldSec != properties.TimeSeconds)
    {
        TW_LOG(TW_INFO, "Last Reading Taken at: %dH  %dM  %dS",properties.TimeHours,properties.TimeMinutes,properties.TimeSeconds);
    }

    /* Update the properties on the server */
    sendPropertyUpdate();
}

/*****************
Property Handler Callbacks 
******************/
enum msgCodeEnum propertyHandler(const char * entityName, const char * propertyName,  twInfoTable ** value, char isWrite, void * userdata) {
    char * asterisk = "*";
    if (!propertyName) propertyName = asterisk;
    TW_LOG(TW_TRACE,"propertyHandler - Function called for Entity %s, Property %s", entityName, propertyName);
    if (value) {
        if (isWrite && *value) {
            /* Property Writes */
            if (strcmp(propertyName, "LedOn") == 0) twInfoTable_GetBoolean(*value, propertyName, 0, &properties.LedOn);
            else if (strcmp(propertyName, "Led2On") == 0) twInfoTable_GetBoolean(*value, propertyName, 0, &properties.Led2On);
            else return TWX_NOT_FOUND;
            return TWX_SUCCESS;
        } else {
            /* Property Reads */
            if (strcmp(propertyName, "PowerSupply") == 0) *value = twInfoTable_CreateFromNumber(propertyName, properties.PowerSupply);
            else if (strcmp(propertyName, "LedOn") == 0) *value = twInfoTable_CreateFromBoolean(propertyName, properties.LedOn);
            else if (strcmp(propertyName, "Led2On") == 0) *value = twInfoTable_CreateFromBoolean(propertyName, properties.Led2On);
            else if (strcmp(propertyName, "Temperature") == 0) *value = twInfoTable_CreateFromInteger(propertyName, properties.Temperature);
            else if (strcmp(propertyName, "TimeHours") == 0) *value = twInfoTable_CreateFromInteger(propertyName, properties.TimeHours);
            else if (strcmp(propertyName, "TimeMinutes") == 0) *value = twInfoTable_CreateFromInteger(propertyName, properties.TimeMinutes);
            else if (strcmp(propertyName, "TimeSeconds") == 0) *value = twInfoTable_CreateFromInteger(propertyName, properties.TimeSeconds);
            else return TWX_NOT_FOUND;
        }
        return TWX_SUCCESS;
    } else {
        TW_LOG(TW_ERROR,"propertyHandler - NULL pointer for value");
        return TWX_BAD_REQUEST;
    }
}

/*****************
Service Callbacks 
******************/
// SwitchLed - Switches led state to On or Off
enum msgCodeEnum SwitchLed(const char * entityName, const char * serviceName, twInfoTable * params, twInfoTable ** content, void * userdata)
{

    if(properties.LedOn == FALSE)// Led is off, turn it on
    {
        ledOnRead = 1;
        ecode = IODBWrite(DISCRETE_OUT, 1, 1, (void *)&ledOnRead, NULL);
    }
    else // turn  led off
    {
        ledOnRead = 0;
        ecode = IODBWrite(DISCRETE_OUT, 1, 1, (void *)&ledOnRead, NULL);
    }

    if(ecode != 0)
    {
        TW_LOG(TW_ERROR,"IODB ERROR: %d",ecode);
        return TWX_INTERNAL_SERVER_ERROR;
    }
    else
        return TWX_SUCCESS;
}



/*******************************************/
/*         Bind Event Callback             */
/*******************************************/
void BindEventHandler(char * entityName, char isBound, void * userdata) {
    if (isBound) TW_LOG(TW_FORCE,"BindEventHandler: Entity %s was Bound", entityName);
    else TW_LOG(TW_FORCE,"BindEventHandler: Entity %s was Unbound", entityName);
}

/*******************************************/
/*    OnAuthenticated Event Callback       */
/*******************************************/
void AuthEventHandler(char * credType, char * credValue, void * userdata) {
    if (!credType || !credValue) return;
    TW_LOG(TW_FORCE,"AuthEventHandler: Authenticated using %s = %s.  Userdata = 0x%x", credType, credValue, userdata);
    /* Could do a delayed bind here */
    /* twApi_BindThing(thingName); */
}


/* ****************
 * Set ThingName with <model>-<serial>
 * ***************/
void setThingName()
{
    char model[32];
    char serial[32];
    FILE *fp;

    // save serail and model to file
    system("serialno > /home/rlcuser/serial.txt");
    system("modelno > /home/rlcuser/model.txt");


    fp = fopen("/home/rlcuser/serial.txt", "r");
    fscanf(fp, "%s", serial);
    fclose(fp);

    // read model
    fp = fopen("/home/rlcuser/model.txt", "r");
    fscanf(fp, "%s", model);
    fclose(fp);

    // clean up
    system("rm -f /home/rlcuser/serial.txt");
    system("rm -f /home/rlcuser/model.txt");

    sprintf(thingName,"%s-%s",model,serial);

}

/***************
Main Loop
****************/
/*
Solely used to instantiate and configure the API.
*/
int main( int argc, char** argv ) {

#if defined NO_TLS
    int16_t port = 80;
#else
    int16_t port = 443;
#endif
    //twDataShape * ds = 0;
    int err = 0;

#ifndef ENABLE_TASKER
    DATETIME nextDataCollectionTime = 0;
#endif

    twLogger_SetLevel(TW_INFO);
    twLogger_SetIsVerbose(1);
    TW_LOG(TW_FORCE, "Starting up");

    /* Get configs */
    tw_configs configs;
    loadConfigsFile("/etc/jbm/sdk.conf",&configs);
    TW_LOG(TW_INFO,"Host: %s",configs.host);
    TW_LOG(TW_INFO,"AppKey: %s",configs.appKey);


    setThingName();

    /* Initialize the API */
    err = twApi_Initialize(configs.host, port, TW_URI, configs.appKey, NULL, MESSAGE_CHUNK_SIZE, MESSAGE_CHUNK_SIZE, TRUE);
    if (err) {
        TW_LOG(TW_ERROR, "Error initializing the API");
        exit(err);
    }

    //Set Initaial Values

    properties.Led2On == FALSE;

    /* Allow self signed certs */
    twApi_SetSelfSignedOk();



    /*Register services */
    twApi_RegisterService(TW_THING, thingName, "SwitchLed", NULL, NULL, TW_NOTHING, NULL, SwitchLed, NULL);

    /* Regsiter our properties */
    twApi_RegisterProperty(TW_THING, thingName, "PowerSupply", TW_NUMBER, NULL, "ALWAYS", 0, propertyHandler, NULL);
    twApi_RegisterProperty(TW_THING, thingName, "Temperature", TW_INTEGER, NULL, "ALWAYS", 0, propertyHandler, NULL);
    twApi_RegisterProperty(TW_THING, thingName, "TimeHours", TW_INTEGER, NULL, "ALWAYS", 0, propertyHandler, NULL);
    twApi_RegisterProperty(TW_THING, thingName, "TimeMinutes", TW_INTEGER, NULL, "ALWAYS", 0, propertyHandler, NULL);
    twApi_RegisterProperty(TW_THING, thingName, "TimeSeconds", TW_INTEGER, NULL, "ALWAYS", 0, propertyHandler, NULL);
    twApi_RegisterProperty(TW_THING, thingName, "Potentiameter", TW_NUMBER, NULL, "ALWAYS", 0, propertyHandler, NULL);
    twApi_RegisterProperty(TW_THING, thingName, "LedOn", TW_BOOLEAN, NULL, "ALWAYS", 0, propertyHandler, NULL);
    twApi_RegisterProperty(TW_THING, thingName, "Led2On", TW_BOOLEAN, NULL, "ALWAYS", 0, propertyHandler, NULL);


    /* Register an authentication event handler */
    twApi_RegisterOnAuthenticatedCallback(AuthEventHandler, NULL); /* Callbacks only when we have connected & authenticated */

    /* Register a bind event handler */
    twApi_RegisterBindEventCallback(thingName, BindEventHandler, NULL); /* Callbacks only when thingName is bound/unbound */
    /* twApi_RegisterBindEventCallback(NULL, BindEventHandler, NULL); *//* First NULL says "tell me about all things that are bound */

    /* Bind our thing - Can bind before connecting or do it when the onAuthenticated callback is made.  Either is acceptable */
    twApi_BindThing(thingName);

    /* Connect to server */
    err = twApi_Connect(CONNECT_TIMEOUT, twcfg.connect_retries);
    if (!err) {
        /* Register our "Data collection Task" with the tasker */
        twApi_CreateTask(DATA_COLLECTION_RATE_MSEC, dataCollectionTask);
    } else {
        TW_LOG(TW_ERROR,"main: Server connection failed after %d attempts.  Error Code: %d", twcfg.connect_retries, err);
    }

    while(1) {
        char in = 0;
#ifndef ENABLE_TASKER
        DATETIME now = twGetSystemTime(TRUE);
        twApi_TaskerFunction(now, NULL);
        twMessageHandler_msgHandlerTask(now, NULL);
        if (twTimeGreaterThan(now, nextDataCollectionTime)) {
            dataCollectionTask(now, NULL);
            nextDataCollectionTime = twAddMilliseconds(now, DATA_COLLECTION_RATE_MSEC);
        }
#else
        //		in = getch();
        //		if (in == 'q') break;
        //		else printf("\n");
        //                printf("\n");
#endif
        twSleepMsec(5);
    }
    twApi_UnbindThing(thingName);
    twSleepMsec(100);
    /*
        twApi_Delete also cleans up all singletons including
        twFileManager, twTunnelManager and twLogger.
        */
    twApi_Delete();
    exit(0);
}
