/*
 * Example implementation of iodb functions
 *
 * Copyright 2016 Red Lion Controls, Inc.  All rights reserved.
 */
/* -*- mode:c;  c-basic-offset: 4; tab-width: 8; -*- vi: set sw=4 ts=8:*/
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <unistd.h>
#include <iodb.h>

/* Types */
#define INT16   short
#define INT32   long
#define UINT16  unsigned short
#define UINT32  unsigned long
#define ULONG   unsigned long
#define BYTE    unsigned char
#define BOOL    int

/* IODB Types */
#define ANALOG_IN       0
#define ANALOG_OUT      1
#define DISCRETE_IN     10
#define DISCRETE_OUT    11
#define LONG_IN         20
#define LONG_OUT        21
#define FLOAT_IN        22
#define FLOAT_OUT       23


int main()
{
    int i;
    UINT16 addr;         /* Starting Register */
    UINT16 regs;         /* Number of Register */
    UINT16 datatype;     /* Type of I/O */
    UINT16 typesize;
    UINT16 typenum;
    INT16 ecode;
    INT16 a_value;
    INT16 analog_values[8];
    INT16 discrete_values;

    /*
     * Write 1 analog value to analog output register 0.
     * Then read the value back in...
     */
    addr = 0;
    regs = 1;
    a_value = 37;
    printf("\nWriting 1 analog, Reading 1 back...\n");
    printf("a_value: %d\n", a_value);
    ecode = IODBWrite(ANALOG_OUT, addr, regs, (void *)&a_value, NULL);
    a_value = 0;
    ecode = IODBRead(ANALOG_OUT, addr, regs, (void *)&a_value, NULL);
    printf("a_value (returned): %d\n", a_value);

    /*
     * Write 3 analog values to 3 consecutive registers,
     * starting at analog output register 1.
     * Then read the values back from register 0.
     */
    addr = 1;
    regs = 3;
    analog_values[0] = 11;
    analog_values[1] = 22;
    analog_values[2] = 33;
    printf("\nWriting 3 analogs, Reading 4 back...\n");
    for (i = 0; i < regs; i++) {
        printf("analog_values[%d]: %d\n", i, analog_values[i]);
    }
    ecode = IODBWrite(ANALOG_OUT, addr, regs, (void *)&analog_values, NULL);

    memset((void *)&analog_values, '0', sizeof(INT16) * 8);
    addr = 0;
    regs = 4;
    ecode = IODBRead(ANALOG_OUT, addr, regs, (void *)&analog_values, NULL);
    for (i = 0; i < regs; i++) {
        printf("analog_values[%d]: %d\n", i, analog_values[i]);
    }

    /*
     * Write 1 discrete value to discrete output register 0.
     * Then read the value back in...
     */
    addr = 0;
    regs = 1;
    discrete_values = 1;  /* Turning discrete ON */
    printf("\nWriting 1 discrete, Reading 1 back...\n");
    printf("discrete_values: %d\n", discrete_values);
    ecode = IODBWrite(DISCRETE_OUT, addr, regs, (void *)&discrete_values, NULL);
    discrete_values = 0;
    ecode = IODBRead(DISCRETE_OUT, addr, regs, (void *)&discrete_values, NULL);
    printf("discrete_values (returned): %d\n", discrete_values);

    /*
     * Write 4 discrete value to discrete output register 4.
     * Then read 8 discretes back, starting at register 0...
     *    D0 = ON  (Previous Write)
     *    D4 = ON
     *    D6 = ON
     *    If nothing else is ON, should return 81.
     */
    addr = 4;
    regs = 4;
    discrete_values = 5;  /* D0 & D2 are on, D1 & D3 are off */
    printf("\nWriting 4 discretes, Reading 8 back...\n");
    printf("discrete_values: %d\n", discrete_values);
    ecode = IODBWrite(DISCRETE_OUT, addr, regs, (void *)&discrete_values, NULL);
    discrete_values = 0;
    addr = 0;
    regs = 8;
    ecode = IODBRead(DISCRETE_OUT, addr, regs, (void *)&discrete_values, NULL);
    printf("discrete_values (returned): %d\n", discrete_values);


    /*
     * Reset Digital outs to 0
     *
     */
    addr = 0;
    regs = 3;
    discrete_values = 0;
    ecode = IODBWrite(DISCRETE_OUT, addr, regs, (void *)&discrete_values, NULL);


    /*
     * Print current SDK Application time to screen every 5 seconds while app is running.
     */
    int M,D,Y,h,m,s;
    FILE *fp;
    while(1)
    {
        // Read current date/time
        M = D = Y = h = m = s = 0;

        IODBRead(ANALOG_OUT, 1010, 1, (void *)&Y, NULL); // Year
        IODBRead(ANALOG_OUT, 1011, 1, (void *)&M, NULL); // Month
        IODBRead(ANALOG_OUT, 1012, 1, (void *)&D, NULL); // Day
        IODBRead(ANALOG_OUT, 1015, 1, (void *)&h, NULL); // Hour
        IODBRead(ANALOG_OUT, 1016, 1, (void *)&m, NULL); // Minute
        IODBRead(ANALOG_OUT, 1017, 1, (void *)&s, NULL); // Second

        // Write time to local file
        fp = fopen("/tmp/sdk.txt", "w");
        fprintf(fp, "SDK Application last run at: %d/%d/%d %d:%d:%d \n",M,D,Y,h,m,s);
        printf("SDK Application last run at: %d/%d/%d %d:%d:%d \n",M,D,Y,h,m,s);
        fclose(fp);

        // Stop the application if first word in the config file says "Stop"
        char buff[255];
        fp = fopen("/etc/jbm/sdk.conf", "r");

        fscanf(fp, "%s", buff);
        if(strcmp(buff, "Stop") == 0)
        {
            break;
        }
        fclose(fp);

        sleep(5);
    }


    exit (0);
}

