/*
 * Template file for reading and writing to iodb
 *
 * Copyright 2016 Red Lion Controls, Inc.  All rights reserved.
 */

#include<unistd.h>
#include<iodb.h>

/* Types */
#define INT16   short
#define INT32   long
#define UINT16  unsigned short
#define UINT32  unsigned long
#define ULONG   unsigned long
#define BYTE    unsigned char
#define BOOL    int

/* IODB Types */
#define ANALOG_IN       0
#define ANALOG_OUT      1
#define DISCRETE_IN     10
#define DISCRETE_OUT    11
#define LONG_IN         20
#define LONG_OUT        21
#define FLOAT_IN        22
#define FLOAT_OUT       23

int EvalDiscrete(const INT16 *bit_packed_array, int bit_location);
void SetDiscrete(INT16 *bit_packed_array, int bit_location, int value);

int main(int argc, char * argv[])
{
    int g_i_debugMde = 0;
    char * l_sp_argPtr;
    for ( int i=1; i<argc; i++)
    {
        l_sp_argPtr = argv[i];
 
        if ( '-' == *l_sp_argPtr)  //switch to the next character
        {
            l_sp_argPtr++;
 
            if ( 'd' == *l_sp_argPtr )       //the -d debug switch
            {
                g_i_debugMode++;
                continue;
            }
        }
    }
    if (!g_i_debugMode)
    {
        //Daemonize
        daemon(1,0);
    }
    INT16 addr;         /* Starting Register */
    INT16 regs;         /* Number of Register */
    INT16 datatype;     /* Type of I/O */
    INT16 ecode;
    INT16 analog_values_in[1024];
    INT16 analog_values_out[1024];
    INT16 discrete_values_in[64];   /* 64 = 1024 / 16 */
    INT16 discrete_values_out[64];
    float float_values_in[1024];
    float float_values_out[1024];  
    long long_values_in[1024];
    long long_values_out[1024];

    while (1)
    {
        // Read 1024 registers of each iodb type
        addr = 0;
        regs = 1024;
        ecode = IODBRead(ANALOG_IN, addr, regs, (void *)&analog_values_in, NULL);
        ecode = IODBRead(ANALOG_OUT, addr, regs, (void *)&analog_values_out, NULL);
        ecode = IODBRead(DISCRETE_IN, addr, regs, (void *)&discrete_values_in, NULL);
        ecode = IODBRead(DISCRETE_OUT, addr, regs, (void *)&discrete_values_out, NULL);
        ecode = IODBRead(FLOAT_IN, addr, regs, (void *)&float_values_in, NULL);
        ecode = IODBRead(FLOAT_OUT, addr, regs, (void *)&float_values_out, NULL);
        ecode = IODBRead(LONG_IN, addr, regs, (void *)&long_values_in, NULL);
        ecode = IODBRead(LONG_OUT, addr, regs, (void *)&long_values_out, NULL);

        // Do Logic 
        if (EvalDiscrete(discrete_values_in,0) == 1 &&
        EvalDiscrete(discrete_values_in,1) == 1)
        {
            SetDiscrete(discrete_values_out,3,1);
        }

        if (EvalDiscrete(discrete_values_in,1) == 1
            &&
            EvalDiscrete(discrete_values_in,2) == 1)
        {
            analog_values_out[0]=1234;
        }

        // Write changes to *_values_out
        ecode = IODBWrite(ANALOG_OUT, addr, regs, (void *)&analog_values_out, NULL);
        ecode = IODBWrite(DISCRETE_OUT, addr, regs, (void *)&discrete_values_out, NULL);
        ecode = IODBWrite(FLOAT_OUT, addr, regs, (void *)&float_values_out, NULL);
        ecode = IODBWrite(LONG_OUT, addr, regs, (void *)&long_values_out, NULL);

        usleep(100000); // 100ms
    }
    return 0;
}

/* Evaluate a Discrete Array from IODB that is bit packed.  

   INT16 *bit_packed_array // The array returned from IODB functions
   int bit_location // The bit (address) location into the array, zero-based.

   Returns a 1 if bit is set
   Returns a 0 if bit is not set
*/
int EvalDiscrete(const INT16 *bit_packed_array, int bit_location)
{
    int word_index = bit_location / 16;
    int offset = bit_location % 16;
    INT16 mask = 0x01 << offset;
    if ((bit_packed_array[word_index] & mask) != 0)
    {
        return 1;
    }
    return 0;
}

/* Set a bit position in a Discrete Array from IODB that is bit packed

   INT16 *bit_packed_array // The array returned from IODB functions
   int bit_location // The bit (address) location into the array, zero-based.
   int value // 0 = clear bit, non-zero = set bit

   Returns nothing.
*/
void SetDiscrete(INT16 *bit_packed_array, int bit_location, int value)
{
    int word_index = bit_location / 16;
    int offset = bit_location % 16;
    INT16 mask = 0x01 << offset;

    if (value)
    {
        bit_packed_array[word_index] |= mask;
    }
    else
    {
        bit_packed_array[word_index] &= ~mask;
    }

    return;
}


