/* -*- mode:c;  c-basic-offset: 4; tab-width: 8; -*- vi: set sw=4 ts=8:*/
/** 
 * @file
 * Declaration of Sixnet\Red Lion Controls SN-Series IOCTRL functions
 *
 * Copyright 2016 Red Lion Controls, Inc..  All rights reserved.
 */

#ifndef SNHW_H
#define SNHW_H

#define BT_GPIO_MINOR   3

/* Same as  TIOCMxxx define */
#define BT_IOCGET       0x5415
#define BT_IOCBIS       0x5416
#define BT_IOCBIC       0x5417
#define BT_IOCSET       0x5418
#define BT_REGGET       0x4201

/* Other I/O (GPIO) Bit Field (User Space Interface) */
#define BT_IN1          (1<<0)
#define BT_OUT1         (1<<16)

/* Defines MAX path and IO */
#define  HWCONFIG_MAX_PATH (512)
#define  HWCONFIG_MAX_IO   (1)

/* Number of IO  interfaces */
unsigned char ioNum;                  

/* Paths and other control information */
char          name[32];
char          ioPath[HWCONFIG_MAX_PATH];   
char          driverPath[HWCONFIG_MAX_PATH]; 
char          controlDevicePath[HWCONFIG_MAX_PATH];
char          adcControlDevicePath[HWCONFIG_MAX_PATH];

/* Structure used to control IO */
unsigned char hw_analogInputs;        /* Number of analog inputs (AI1) */
unsigned char hw_digitalInputs;       /* Number of digital inputs (DI1) */
unsigned char hw_digitalOutputs;      /* Number of digital outputs (DO1) */

/* Set the modem I/O driverPath, controlDevicePath, adcControlDevicePath
 * and defines number of available modules (DI, DO, AI). 
 */
void hwConfig_SetModemModel();
  
/* Perform an IOCTL action on an open file descriptor function */
int hwControl_ioctl(int controlFd, int controlAction, int * controlBits, char * controlPath, int debug);

/* SET a Discrete Output function */
int hwControl_Io_setDO(int value, int debug);

/* GET a Discrete Input function */
int hwControl_Io_getDI(int * p_value, int debug);

/* GET a Discrete Output function */
int hwControl_Io_getDO(int * p_value, int debug);

/* GET a Analog Input function */
int hwControl_Io_getAI(double * p_value, int debug);

/* Open control path function */
int hwControl_Io_openControl(int  debug, int  * p_controlFd, char ** p_controlPath);
#endif
 
