/*
 *  Copyright (C) 2015 ThingWorx Inc.
 *
 *  Error code definitions
 */

/**
 * \file twErrors.h
 * \brief ThingWorx C SDK error code definitions
 *
*/

#ifndef TW_ERRORS_H
#define TW_ERRORS_H

#define TW_OK 0

/**
 * \name General Errors 1xx
*/
#define TW_UNKNOWN_ERROR 100           /**< Unknown error **/
#define TW_INVALID_PARAM 101           /**< Invalid parameter **/
#define TW_ERROR_ALLOCATING_MEMORY 102 /**< Error allocating memory **/
#define TW_ERROR_CREATING_MTX 103      /**< Error creating mutex **/
#define TW_ERROR_WRITING_FILE 104
#define TW_ERROR_READING_FILE 105

/**
 * \name Websocket Errors 2xx
*/
#define TW_UNKNOWN_WEBSOCKET_ERROR 200
#define TW_ERROR_INITIALIZING_WEBSOCKET 201
#define TW_TIMEOUT_INITIALIZING_WEBSOCKET 202 /**< Timeout initializing websocket **/
#define TW_WEBSOCKET_NOT_CONNECTED 203
#define TW_ERROR_PARSING_WEBSOCKET_DATA 204
#define TW_ERROR_READING_FROM_WEBSOCKET 205
#define TW_WEBSOCKET_FRAME_TOO_LARGE 206
#define TW_INVALID_WEBSOCKET_FRAME_TYPE 207
#define TW_WEBSOCKET_MSG_TOO_LARGE 208
#define TW_ERROR_WRITING_TO_WEBSOCKET 209
#define TW_INVALID_ACCEPT_KEY 210

/**
 * \name Messaging Errors 3xx
*/
#define TW_NULL_OR_INVALID_MSG_HANDLER 300
#define TW_INVALID_CALLBACK_STRUCT 301
#define TW_ERROR_CALLBACK_NOT_FOUND 302
#define TW_INVALID_MSG_CODE 303
#define TW_INVALID_MSG_TYPE 304
#define TW_ERROR_SENDING_MSG 305
#define TW_ERROR_WRITING_OFFLINE_MSG_STORE 306
#define TW_ERROR_MESSAGE_TOO_LARGE 307
#define TW_WROTE_TO_OFFLINE_MSG_STORE 308
#define TW_INVALID_MSG_STORE_DIR 309
#define TW_MSG_STORE_FILE_NOT_EMPTY 310

/**
 * \name ::twPrimitive/::twInfoTable Errors 4xx
*/
#define TW_ERROR_ADDING_DATASHAPE_ENTRY 400
#define TW_INDEX_NOT_FOUND 401
#define TW_ERROR_GETTING_PRIMITIVE 402
#define TW_INVALID_BASE_TYPE 403

/**
 * \name List Errors 5xx
*/
#define TW_LIST_ENTRY_NOT_FOUND 500

/**
 * \name API Errors 6xx
*/
#define TW_NULL_OR_INVALID_API_SINGLETON 600
#define TW_ERROR_SENDING_RESP 601
#define TW_INVALID_MSG_BODY 602
#define TW_INVALID_MSG_PARAMS 603
#define TW_INVALID_RESP_MSG 604
#define TW_NULL_API_SINGLETON 605
#define TW_ERROR_CREATING_MSG 606
#define TW_ERROR_INITIALIZING_API 607

/**
 * \name Tasker Errors 7xx
*/
#define TW_MAX_TASKS_EXCEEDED 700
#define TW_TASK_NOT_FOUND 701

/**
 * \name Logger Errors 8xx
*/
#define TW_NULL_OR_INVALID_LOGGER_SINGLETON 800

/**
 * \name Utils Errors 9xx
*/
#define TW_BASE64_ENCODE_OVERRUN 900
#define TW_BASE64_DECODE_OVERRUN 901

/**
 * \name System Socket Errors 10xx
*/
#define TW_ERROR_WRITING_TO_SOCKET 1000
#define TW_SOCKET_INIT_ERROR 1001
#define TW_INVALID_SSL_CERT 1002
#define TW_SOCKET_NOT_FOUND 1003
#define TW_HOST_NOT_FOUND 1004
#define TW_ERROR_CREATING_SSL_CTX 1005
#define TW_ERROR_CONNECTING_TO_PROXY 1006
#define TW_TIMEOUT_READING_FROM_SOCKET 1007
#define TW_ERROR_READING_RESPONSE 1008
#define TW_INVALID_PROXY_CREDENTIALS 1009
#define TW_UNSUPPORTED_PROXY_AUTH_TYPE 1010
#define TW_ENABLE_FIPS_MODE_FAILED 1011
#define TW_FIPS_MODE_NOT_SUPPORTED 1012
/**
 * \name msgCodeEnum Errors 11xx
*/
#define TW_BAD_REQUEST 1100
#define TW_UNAUTHORIZED 1101
#define TW_ERROR_BAD_OPTION 1102
#define TW_FORBIDDEN 1103
#define TW_NOT_FOUND 1104
#define TW_METHOD_NOT_ALLOWED 1105
#define TW_NOT_ACCEPTABLE 1106
#define TW_PRECONDITION_FAILED 1107
#define TW_ENTITY_TOO_LARGE 1108
#define TW_UNSUPPORTED_CONTENT_FORMAT 1109
#define TW_INTERNAL_SERVER_ERROR 1110
#define TW_NOT_IMPLEMENTED 1111
#define TW_BAD_GATEWAY 1112
#define TW_SERVICE_UNAVAILABLE 1113
#define TW_GATEWAY_TIMEOUT 1114

/**
 * \name Managed Property Errors 12xx
*/
#define TW_SUBSCRIBEDPROP_MGR_NOT_INTIALIZED 1200
#define TW_SUBSCRIBED_PROPERTY_NOT_FOUND 1201

/**
 * \name File Transfer Errors 13xx
*/
#define TW_FILE_XFER_MANAGER_NOT_INITIALIZED 1300
#define TW_ERROR_CREATING_STAGING_DIR 1301
#define TW_FILE_NOT_FOUND 1302
#define FILE_TRANSFER_FAILED 1303

/**
 * \name Tunneling Errors 14xx
*/
#define TW_TUNNEL_MANAGER_NOT_INITIALIZED 1400
#define TW_TUNNEL_CREATION_FAILED 1401

#endif
