/***************************************
 *  Copyright (C) 2015 ThingWorx Inc.  *
 ***************************************/

/**
 * \file twAxTLS.h
 * \brief Portable ThingWorx AxTLS wrapper layer
*/

#ifndef AXTLS_H
#define AXTLS_H

#include "twOSPort.h"
#include "stringUtils.h"

#include "stdio.h"
#include "string.h"

#if defined WIN32
#define STDCALL                 __stdcall
#endif
#include "ssl.h"

#ifdef __cplusplus
extern "C" {
#endif

int16_t returnValue(int16_t v);
typedef struct ssl_struct {
	SSL * ssl;
	uint8_t * leftover_buf;
	uint8_t * leftover_ptr;
	uint16_t leftover_cnt;
	uint16_t max_leftover;
} ssl_struct;

#define TW_SSL_CTX						SSL_CTX
#define TW_SSL							ssl_struct
#define TW_SSL_SESSION_ID_SIZE			SSL_SESSION_ID_SIZE
#define TW_SSL_SESSION_ID(a)            a ? a->ssl->session_id : NULL
#define TW_GET_CERT_SIZE				ssl_get_config(SSL_MAX_CERT_CFG_OFFSET)
#define TW_GET_CA_CERT_SIZE				ssl_get_config(SSL_MAX_CA_CERT_CFG_OFFSET)
#define TW_NEW_SSL_CTX					ssl_ctx_new(options | SSL_SERVER_VERIFY_LATER, SSL_DEFAULT_CLNT_SESS)
/*#define TW_NEW_SSL_CLIENT(a,b,c,d)		ssl_client_new(a,b,c,d)
#define TW_NEW_SERVER(a, b)				ssl_server_new(a,b)	
#define TW_HANDSHAKE_SUCCEEDED(a)		(a ? (ssl_handshake_status(a->ssl)) == SSL_OK) : FALSE)
#define TW_SSL_FREE(a)					ssl_free(a)*/
#define TW_SSL_CTX_FREE(a)				ssl_ctx_free(a)
#define TW_SSL_WRITE(a,b,c)				ssl_write(a->ssl,(uint8_t *)b,c)
#define TW_USE_CERT_FILE(a,b,c)			ssl_obj_load(a, SSL_OBJ_X509_CERT, b, NULL)
#define TW_USE_KEY_FILE(a,b,c,d)		ssl_obj_load(a, SSL_OBJ_RSA_KEY, b, d)
#define TW_USE_CERT_CHAIN_FILE(a,b,c)	ssl_obj_load(a, SSL_OBJ_X509_CACERT, b, NULL)
#define TW_SET_CLIENT_CA_LIST(a,b)		ssl_obj_load(a, SSL_OBJ_X509_CACERT, (const char *)b, NULL)
#define TW_ENABLE_FIPS_MODE(a)			returnValue(TW_FIPS_MODE_NOT_SUPPORTED)
#define TW_SHA1_CTX						SHA1_CTX
#define TW_SHA1_INIT(a)					SHA1_Init(a)
#define TW_SHA1_UPDATE(a,b,c)			SHA1_Update(a,b,c)
#define TW_SHA1_FINAL(a,b)				SHA1_Final(a,b)
#define TW_MD5_CTX						MD5_CTX
#define TW_MD5_INIT(a)					MD5_Init(a)
#define TW_MD5_UPDATE(a,b,c)			MD5_Update(a,b,c)
#define TW_MD5_FINAL(a,b)				MD5_Final(a,b)
#define DATA_AVAILABLE(a,b,c)           1
  
static INLINE TW_SSL * TW_NEW_SSL_CLIENT(SSL_CTX *ssl_ctx, twSocket * client_fd, void * session_id, uint8_t sess_id_size) {
	TW_SSL * s = (TW_SSL *)TW_CALLOC(sizeof(TW_SSL), 1);
	if (!s) return NULL;
	s->ssl = ssl_client_new(ssl_ctx, client_fd, (const uint8_t *)session_id, sess_id_size);
	if (!s->ssl) {
		TW_FREE(s);
		s = NULL;
	}
	return s;
}

static INLINE char TW_HANDSHAKE_SUCCEEDED(TW_SSL * s) {
	if (!s || !s->ssl) return FALSE;
	return (ssl_handshake_status(s->ssl) == SSL_OK);
}

static INLINE TW_SSL * TW_NEW_SERVER(SSL_CTX *ssl_ctx, twSocket * client_fd) {
	TW_SSL * s = (TW_SSL *)TW_CALLOC(sizeof(TW_SSL), 1);
	if (!s) return NULL;
	s->ssl = ssl_server_new(ssl_ctx, client_fd);
	if (!s->ssl) {
		TW_FREE(s);
		s = NULL;
	}
	return s;
}

static INLINE void TW_SSL_FREE(TW_SSL * s){ 
	if(s) {
		if (s->ssl) ssl_free(s->ssl);
		if (s->leftover_buf) TW_FREE(s->leftover_buf);
		TW_FREE(s);
	}
}

/**
 * \brief Tries to read \p len bytes from the specified \p ssl into \p buf.
 *
 * \param[in]     ssl       The TLS/SSL connection to read from.
 * \param[out]    buf       A buffer to store the read data.
 * \param[in]     len       The number of bytes to read.
 * \param[in]     timeout   The amount of time (in milliseconds) to wait for
 *                          I/O to be unblocked.
 *
 * \return The number of bytes read or -1 if an error was encountered.
*/
static INLINE int TW_SSL_READ(TW_SSL * ssl, char * buf, int len, int32_t timeout) { 
	int ret = -1; 
	uint8_t *read_buf = NULL;
	DATETIME maxtime = twGetSystemMillisecondCount() + timeout;
    if(!ssl || !buf) return -1;
	/* Anything left from last read? */
	if (ssl->leftover_cnt && ssl->leftover_ptr) {
		uint16_t bytes = ssl->leftover_cnt > len ? len : ssl->leftover_cnt;
		memcpy(buf, ssl->leftover_ptr, bytes);
		if (ssl->leftover_cnt > len) {
			ssl->leftover_cnt -= len;
			ssl->leftover_ptr += bytes;
		} else {
			ssl->leftover_cnt = 0;
			ssl->leftover_ptr = ssl->leftover_buf;
		}
		/*** printf("\n ############# AXTLS:SSL_READ - Reading from leftover data.  Asked for %d, Returned %d\n\n", len, bytes);  ***/
		return bytes;
	}
	/* Loop until we are unblocked or timeout */
	while (twTimeGreaterThan(maxtime, twGetSystemMillisecondCount()) && (ret = ssl_read(ssl->ssl, &read_buf)) == SSL_OK) {

	}
	if (ret > SSL_OK) {
		uint16_t bytes = ret > len ? len : ret;
		memcpy(buf, read_buf, bytes);
		ssl->leftover_cnt = ret > len ? ret - len : 0;
		/*** printf("\n ############# AXTLS:SSL_READ - Asked for %d bytes, Read %d bytes\n\n", len, ret); ***/
		if (ssl->leftover_cnt) {
			if (ssl->leftover_cnt > ssl->max_leftover) {
				if (ssl->leftover_buf) TW_FREE(ssl->leftover_buf);
				/*** printf("\n ############# AXTLS:SSL_READ - Allocating %d bytes for leftover buffer\n\n", ssl->leftover_cnt); ***/
				ssl->leftover_buf = (uint8_t *)TW_CALLOC(ssl->leftover_cnt,1);
				if (!ssl->leftover_buf) {
					printf("\nAXTLS - PANIC! Unable to allocate memory\n\n");
					return -1;
				}
				ssl->leftover_ptr = ssl->leftover_buf;
				ssl->max_leftover = ssl->leftover_cnt;
			}
			memcpy(ssl->leftover_buf, read_buf + bytes, ssl->leftover_cnt);
			/*** printf("\n ############# AXTLS:SSL_READ - Copied %d bytes to leftover buffer\n\n", ssl->leftover_cnt);  ***/
		}
		return bytes;
	} else if (ret == SSL_OK) {
		/* the WaitFor timed out, meaning we have no more data from the server */
		/*** printf("\n ############# AXTLS:SSL_READ - WaitFor timeout while already begun reading TLS packet\n\n");  ***/
		return TW_READ_TIMEOUT;
	} else return ret; 
}

/**
 * \brief Wait for a TLS/SSL client to initiate a TLS/SSL handshake.
 *
 * \param[in]     s         The TLS/SSL connection to accept.
 *
 * \return 0 on success, -1 on failure.
*/
static int TW_SSL_ACCEPT(TW_SSL * s) {
	if (!s || !s->ssl) return -1;
    while (ssl_read(s->ssl, NULL) == SSL_OK) {
        if (s->ssl->next_state == HS_CLIENT_HELLO)
            return 0;   /* we're done */
    }
    return -1;
}

/**
 * \brief Validates the certificate of a TLS/SSL connection.
 *
 * \param[in]     s             The TSL/SSL connection to validate the certificate of.
 * \param[in]     selfSignedOk  If #TRUE, accept self-signed certificates, if
 *                              #FALSE, don't.
 *
 * \return 0 on success, -1 on failure.
*/
static INLINE int TW_VALIDATE_CERT(TW_SSL * ssl, char selfSignedOk) {
	int res = 1;
	if (!ssl || !ssl->ssl) return 1;
	res = ssl_verify_cert ( ssl->ssl ); 
	if (res) res = res - SSL_X509_OFFSET;
	if (res == SSL_OK) return 0;
	if( res != SSL_OK && (selfSignedOk && res == X509_VFY_ERROR_SELF_SIGNED)) return 0;
	return 1;
}

/**
 * \brief Gets a X509 certificate field of a TLS/SSL connection.
 *
 * \param[in]     s         The TSL/SSL connection to get the 509 certificate
 *                          field of.
 * \param[in]     field     The X509 certificate field to get.  May be
 *                          #SSL_X509_CERT_COMMON_NAME,
 *                          #SSL_X509_CERT_ORGANIZATION,
 *                          #SSL_X509_CERT_ORGANIZATIONAL_NAME,
 *                          #SSL_X509_CA_CERT_COMMON_NAME,
 *                          #SSL_X509_CA_CERT_ORGANIZATION, or
 *                          #SSL_X509_CA_CERT_ORGANIZATIONAL_NAME.
 *
 * \return A string containing the data from the specified \p field.  Returns
 * NULL if an error was encountered.
 *
 * \note Calling function will gain ownership of the returned string and is
 * responsible for freeing it..
*/
static INLINE char * TW_GET_X509_FIELD(TW_SSL * ssl, char field) {
	const char * tmp = NULL;
	if (!ssl || !ssl->ssl) return NULL;
	switch (field) {
	case TW_SUBJECT_CN:
		tmp = ssl_get_cert_dn(ssl->ssl, SSL_X509_CERT_COMMON_NAME);
		break;
	case TW_SUBJECT_O:
		tmp = ssl_get_cert_dn(ssl->ssl, SSL_X509_CERT_ORGANIZATION);
		break;
	case TW_SUBJECT_OU:
		tmp = ssl_get_cert_dn(ssl->ssl, SSL_X509_CERT_ORGANIZATIONAL_NAME);
		break;
	case TW_ISSUER_CN:
		tmp = ssl_get_cert_dn(ssl->ssl, SSL_X509_CA_CERT_COMMON_NAME);
		break;
	case TW_ISSUER_O:
		tmp = ssl_get_cert_dn(ssl->ssl, SSL_X509_CA_CERT_ORGANIZATION);
		break;
	case TW_ISSUER_OU:
		tmp = ssl_get_cert_dn(ssl->ssl, SSL_X509_CA_CERT_ORGANIZATIONAL_NAME);
		break;
	default:
		tmp = NULL;
	}
	return duplicateString(tmp);
}

#ifdef __cplusplus
}
#endif

#endif /* AXTLS_H  */
